// server.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <assert.h>


#define LISTEN_PORT 27015
#define MAX_MSG_LEN 256
#define BACKLOG     10


// server code
// - Initialization:
//   1. create a listening socket
//   2. bind the socket to an address (IP + port)
//   3. put the socket in a listening state
// - Work:
//   4. wait for client's connection
//   5. communicate with the client (send / recv)
//   6. close client socket
//   7. return to wait for client's connection


int _tmain(int argc, _TCHAR* argv[])
{
    _tprintf(_T("Server running...\n"));
	// initiates use of the Winsock DLL by a process

    // In order to support various Windows Sockets implementations and applications 
    // that can have functional differences from the latest version 
    // of Windows Sockets specification, a negotiation takes place in WSAStartup.
    // The caller of WSAStartup passes in the wVersionRequested parameter the highest 
    // version of the Windows Sockets specification that the application supports.
    // The Winsock DLL indicates the highest version of the Windows Sockets specification 
    // that it can support in its response.
    // The current version of the Windows Sockets specification is version 2.2
    WSADATA wsaData;
	int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
	assert(iResult == 0);

	// create a socket for clients connections
	SOCKET hListenSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	assert(hListenSocket != INVALID_SOCKET);

	// prepare address structure to bind listen socket
	sockaddr_in service;
	service.sin_family = AF_INET;
    // convert IP address in "." (dotted) notation to a binary representation
	service.sin_addr.s_addr = inet_addr("132.67.250.232");
    // specify the port to bind to. use htons() to convert from host to network
    // byte order
	service.sin_port = htons(LISTEN_PORT);

	// bind listen socket
	iResult = bind(hListenSocket, (SOCKADDR*)&service, sizeof(service));
	assert(iResult != SOCKET_ERROR);

	// places the socket in a state in which it is listening for an incoming connection.
	iResult = listen(hListenSocket, BACKLOG);
	assert(iResult != SOCKET_ERROR);

	do
	{
		// prepare structure for client (source) address
		sockaddr client_addr;
		int nLen = sizeof(client_addr);
		
		// wait for connection
        // The accept function extracts the first connection on the queue of pending 
        // connections. It then creates and returns a handle to the new socket.
        // The newly created socket is the socket that will handle the actual connection.
		SOCKET hRecvSocket = accept(hListenSocket, &client_addr, &nLen);
		assert(hRecvSocket != INVALID_SOCKET);

		// prepare buffer for incoming data
		char buf[MAX_MSG_LEN];
		int nLeft = sizeof(buf);
		int iPos = 0;

        // loop until all data were read
		do 
		{
            // receives data from the socket 
			int nNumBytes = recv(hRecvSocket, buf + iPos, nLeft, 0);
            assert(nNumBytes != SOCKET_ERROR);

			// check if client closed connection
			if (nNumBytes == 0)
				break;

			// update free space and pointer to next byte
			nLeft -= nNumBytes;
			iPos += nNumBytes;

		} while(TRUE);

		_tprintf(_T("server> %S\n"), (LPSTR)buf);

		// close socket
		closesocket(hRecvSocket);
		
		// check if exit command received
		if (strcmp((LPSTR)buf, "exit") == 0)
			break;

	} while(TRUE);

	// close listen socket
	closesocket(hListenSocket);

	// release windows socket resources
	WSACleanup();

	return 0;
}

